%--------------------------------------------------------------------------	
function ColorMemory()
    prefs        = Preferences();
    prefs.subNum = input('Subject number: ');	
    [win,fid]    = Initialize(prefs); 
	
	% Rectangles for drawing the colors into
    [rectangles] = CreateRectangles(prefs, win);
	[colors]     = GetColors();
    
	% Generate trials for this subject
    [trials]   = SetupTrials(prefs.trialSetSize, prefs.numTrials);
    [practice] = SetupTrials(prefs.trialSetSize, prefs.numPracticeTrials);
    
    % Instructions
    img = imread('instructions.jpg');
    Screen('PutImage', win, img);
    Screen('Flip', win);  KbWait; 
    
    % Practice trials
    for i=1:length(practice)
        DrawCenteredText(win, 'Press any key to start a practice trial');
        Screen('Flip', win);
        WaitSecs(1.0); KbWait; 
                
        DrawStim(win, prefs, practice(i), rectangles, colors);
        GetResponse(win, prefs, practice(i).testWhich, rectangles);
        WaitSecs(prefs.betweenTrials);        
    end

	% For each trial
	for i=1:length(trials)       
        if mod(i-1,prefs.trialsPerBlock)==0
            txt = sprintf('Block %d of %d - Hit any key to continue', ...
                ceil(i/prefs.trialsPerBlock), ceil(length(trials)/prefs.trialsPerBlock));
            DrawCenteredText(win, txt);
            Screen('Flip', win); KbWait;
        end
        
		% Draw stimuli
        DrawStim(win, prefs, trials(i), rectangles, colors);
        
		% Get response 
		[trials(i).selectedWhich, trials(i).rt] = ...
            GetResponse(win, prefs, trials(i).testWhich, rectangles);

		% Save information to file
 		fprintf(fid, '%d\t', trials(i).colorsToShow);
        fprintf(fid, '%d\t%d\t%f\n', trials(i).testWhich, trials(i).selectedWhich, ...
            trials(i).rt);
		WaitSecs(prefs.betweenTrials);
    end

    DrawCenteredText(win, 'Done! Thank you.');
    Screen('Flip', win); KbWait;
            
	% Clean up
    fclose(fid);
    Screen('CloseAll');
	
%--------------------------------------------------------------------------	        
function DrawStim(win, prefs, trial, rectangles, colors)
    % Draw the colors
    for curOb = 1:length(trial.colorsToShow)
        Screen('FillOval', win, colors(trial.colorsToShow(curOb),:), rectangles{curOb});
        Screen('FrameOval', win, prefs.frameOutline, rectangles{curOb}, 1, 1);
    end
    rct = Screen('Rect', win);
    Screen('FillOval', win, prefs.foreColor, CenterRect([0 0 8 8], rct)); 
    [VBLTimestamp]=Screen('Flip', win);

    % Draw blank screen (just circles)
    DrawCircles(win, prefs, rectangles);
    Screen('Flip', win, VBLTimestamp+prefs.timeForColors); 
    WaitSecs(prefs.timeDelay);
        
%--------------------------------------------------------------------------
function [selectedWhich,rt]=GetResponse(win, prefs, whichLoc, rectangles)	
    % Dark circle around the item being tested
    DrawCircles(win, prefs, rectangles);
    Screen('FrameOval', win, prefs.frameOutline, rectangles{whichLoc}, 3, 3);
    Screen('Flip', win);

    % Get key from the subject
    start = GetSecs;	   
    while 1
        [keyIsDown, secs, keyCode] = KbCheck;
        if any(keyCode(prefs.keys))
            rt = secs - start;
            selectedWhich = find(prefs.keys == find(keyCode ~= 0, 1));
            break;
        end
        if keyCode(prefs.quitKey), sca; error('User quit'); end
    end

    % Blank the screen
    DrawCircles(win, prefs, rectangles);   
    Screen('Flip', win);
    
%--------------------------------------------------------------------------	    
function [trials] = SetupTrials(setSize, numTrials)    
    % Generate trials
	for i=1:numTrials
        trials(i).colorsToShow = Shuffle(1:setSize);
        trials(i).testWhich = Randi(setSize);
    end
    
%--------------------------------------------------------------------------	
function p = Preferences()
	% Setup preferences for this experiment
    p.studyName       = 'rememberColors';
    
    % What colors and monitor to use
	p.monitor         = max(Screen('Screens'));
	p.backColor       = [128 128 128];
	p.foreColor       = [0 0 0];
    p.frameOutline    = [0 0 0];
    
    % What keys the subject can use to respond
	p.keys		      = [KbName('a') KbName('s') KbName('d') KbName('f')...
                         KbName('g') KbName('h') KbName('j') KbName('k')];		
	p.quitKey         = KbName('q');
    
    % Information about the location and size of the stimuli
	p.circRad         = 180;
	p.stimSize        = 85;
    
    % How many trials and how many items per trial
	p.trialSetSize    = 8;
	p.numTrials       = 600;
    p.trialsPerBlock  = 60;
    p.numPracticeTrials = 3;
    
    % Timing information
    p.timeForColors   = 1.0;
    p.timeDelay       = 1.0;
    p.betweenTrials   = 1.5;
        
%--------------------------------------------------------------------------        
function [rectangles] = CreateRectangles(prefs, win)
    % Create the rectangles where the circles will go, in a circle around
    % the center of the screen
	rct = Screen('Rect', win);
	curRect = [0 0 prefs.stimSize prefs.stimSize];
	for i=1:prefs.trialSetSize
		locThisX = cosd(360/prefs.trialSetSize .* i) .* prefs.circRad + rct(3)/2;
		locThisY = sind(360/prefs.trialSetSize .* i) .* prefs.circRad + rct(4)/2;	
		rectangles{i} = CenterRectOnPoint(curRect, locThisX, locThisY);
    end
    
%--------------------------------------------------------------------------
function colors = GetColors()
    colors = [1 0 0;  0 1 0;  0 0 1;  1 0 1;  ...
              0 1 1;  1 1 0;  0 0 0;  1 1 1];
    colors = colors .* 256;
    
%--------------------------------------------------------------------------
function DrawCircles(win, prefs, rectangles)
    % Draw an outline of a circle inside each of the rectangles
    rct = Screen('Rect', win);
    Screen('FillRect', win, prefs.backColor);
    for j=1:length(rectangles)
        Screen('FrameOval', win, prefs.frameOutline, rectangles{j}, 1, 1);
    end	
    Screen('FillOval', win, prefs.foreColor, CenterRect([0 0 8 8], rct)); 

%--------------------------------------------------------------------------
function DrawCenteredText(win, text, color, xoff, yoff)
    bbox = Screen('TextBounds', win, text);
    bbox = CenterRect(bbox, Screen('Rect', win));
    x=bbox(RectLeft);
    y=bbox(RectTop);
    Screen('DrawText', win, text, x+0, y-30);
    
 %--------------------------------------------------------------------------	
function [win,fid] = Initialize(prefs)	
	% Do setup for the experiment
	commandwindow;
    rand('state',sum(100*clock));
	win = Screen('OpenWindow', prefs.monitor, prefs.backColor);
	Screen('FillRect', win, prefs.backColor);
	Screen('Flip', win);
	Screen('TextSize', win, 30);
	
	% Setup output file
    if (~exist('Data', 'file'))
        mkdir('Data');
    end
	fName = sprintf('Data/%s_sub%d.dat', prefs.studyName, prefs.subNum);
    while exist(fName,'file')
        prefs.subNum = prefs.subNum+100;
        fName = sprintf('Data/%s_sub%d.dat', prefs.studyName, prefs.subNum);
    end
	fid = fopen(fName, 'w');   
    

	
	
	